#ifndef INC_GTGC_H
#define INC_GTGC_H

#include <X11/Xlib.h>

#include "gtbase.h"
#include "gtdisplay.h"
#include "gtdrawable.h"
#include "gtpixmap.h"
#include "gtfont.h"

/************************************************************************\
*
* GtGC class header.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

class GtGC : public GtBase
{
public:
		/* Standard */
	GtGC(void);
	void Destroy(void);
	void DeInit(void);
	sbool Init(GtDisplay *d,GtTag *tgs);
	sbool Init(GtTag *tgs);
	sbool Set(GtTag *tgs);
	void Get(GtTag *tgs);
	char *GetClass(void) { return("GtFont"); }
	sbool IsSubClassOf(char *cn);
		/* Set values */
	inline void SetForeground(ulong col) { XSetForeground(xdsp,gc,col); }
	inline void SetBackground(ulong col) { XSetBackground(xdsp,gc,col); }
	inline void SetPlaneMask(ulong msk) { XSetPlaneMask(xdsp,gc,msk); }
	inline void SetFunction(int func) { XSetFunction(xdsp,gc,func); }
	inline void SetState(ulong fg,ulong bg,int func,ulong msk)
	{ 
		XSetState(xdsp,gc,fg,bg,func,msk);
	}
	inline void SetArcMode(int am) { XSetArcMode(xdsp,gc,am); }
	inline void SetSubWinMode(int sm) { XSetSubwindowMode(xdsp,gc,sm); }
	inline void SetGraphicsExposures(sbool ge)
	{
		XSetGraphicsExposures(xdsp,gc,(ge ? True : False));
	}
	inline void SetFont(GtFont *fnt) { XSetFont(xdsp,gc,fnt->XFnt()); }
	inline void SetFillStyle(int fs) { XSetFillStyle(xdsp,gc,fs); }
	inline void SetFillRule(int fr) { XSetFillRule(xdsp,gc,fr); }
	inline void SetLineAttributes(unsigned int width,int lstyle,int cstyle,int jstyle)
	{
		XSetLineAttributes(xdsp,gc,width,lstyle,cstyle,jstyle);
	}
	inline void SetDashes(int off,char *dlist,int nr) { XSetDashes(xdsp,gc,off,dlist,nr); }
	inline void SetClipOrigin(int cxp,int cyp) { XSetClipOrigin(xdsp,gc,cxp,cyp); }
	inline void SetClipMask(GtPixmap *pix) { XSetClipMask(xdsp,gc,pix->XPix()); }
	inline void SetClipRectangles(int cxp,int cyp,XRectangle *rects,int nr,int order)
	{
		XSetClipRectangles(xdsp,gc,cxp,cyp,rects,nr,order);
	}
	inline void SetTile(GtPixmap *pix) { XSetTile(xdsp,gc,pix->XPix()); }
	inline void SetStipple(GtPixmap *pix) { XSetStipple(xdsp,gc,pix->XPix()); }
	inline void SetTSOrigin(int txp,int typ) { XSetTSOrigin(xdsp,gc,txp,typ); }
		/* Drawing */
  inline void CopyArea(GtDrawable *s,GtDrawable *d,int sx,int sy,int w,int h,
  								int dx,int dy)
  {
  	XCopyArea(xdsp,s->XDr(),d->XDr(),gc,sx,sy,w,h,dx,dy);
  }
  inline void CopyPlane(GtDrawable *s,GtDrawable *d,int sx,int sy,int w,int h,
  								int dx,int dy,ulong plane)
  {
  	XCopyPlane(xdsp,s->XDr(),d->XDr(),gc,sx,sy,w,h,dx,dy,plane);
  }
  inline void DrawPoint(GtDrawable *d,int x,int y)
  {
  	XDrawPoint(xdsp,d->XDr(),gc,x,y);
  }
  inline void DrawPoints(GtDrawable *d,XPoint *xps,int nr,int md)
  {
  	XDrawPoints(xdsp,d->XDr(),gc,xps,nr,md);
  }
  inline void DrawLine(GtDrawable *d,int x1,int y1,int x2,int y2)
  {
  	XDrawLine(xdsp,d->XDr(),gc,x1,y1,x2,y2);
  }
  inline void DrawLines(GtDrawable *d,XPoint *xps,int nr,int md)
  {
  	XDrawLines(xdsp,d->XDr(),gc,xps,nr,md);
  }
  inline void DrawSegments(GtDrawable *d,XSegment *xseg,int nr)
  {
  	XDrawSegments(xdsp,d->XDr(),gc,xseg,nr);
  }
  inline void DrawRectangle(GtDrawable *d,int x,int y,ulong w,ulong h,sbool fill)
  {
  	if(fill)
  		XFillRectangle(xdsp,d->XDr(),gc,x,y,w,h);
  	else
  		XDrawRectangle(xdsp,d->XDr(),gc,x,y,w,h);
  }
  inline void DrawRectangles(GtDrawable *d,XRectangle *xrc,int nr,sbool fill)
  {
  	if(fill)
  		XFillRectangles(xdsp,d->XDr(),gc,xrc,nr);
  	else
  		XDrawRectangles(xdsp,d->XDr(),gc,xrc,nr);
  }
  inline void DrawArc(GtDrawable *d,int x,int y,ulong w,ulong h,int a1,int a2,sbool fill)
 	{
  	if(fill)
  		XFillArc(xdsp,d->XDr(),gc,x,y,w,h,a1,a2);
  	else
  		XDrawArc(xdsp,d->XDr(),gc,x,y,w,h,a1,a2);
  }
  inline void DrawArcs(GtDrawable *d,XArc *xac,int nr,sbool fill)
 	{
  	if(fill)
  		XFillArcs(xdsp,d->XDr(),gc,xac,nr);
  	else
  		XDrawArcs(xdsp,d->XDr(),gc,xac,nr);
  }
  inline void FillPolygon(GtDrawable *d,XPoint *xps,int nr,int shp,int md)
  {
  	XFillPolygon(xdsp,d->XDr(),gc,xps,nr,shp,md);
  }
  inline void DrawText(GtDrawable *d,int x,int y,XTextItem *xtxt,int nr)
  {
  	XDrawText(xdsp,d->XDr(),gc,x,y,xtxt,nr);
  }
  inline void DrawString(GtDrawable *d,int x,int y,char *str,int len,sbool fill)
  {
  	if(fill)
  		XDrawImageString(xdsp,d->XDr(),gc,x,y,str,len);
  	else
  		XDrawString(xdsp,d->XDr(),gc,x,y,str,len);
  }
		/* Values */
	inline GC XGC(void) { return(gc); }
		/* Friends */
	friend GtGC *GtMkSharedGC(GtDisplay *d,GtTag *tgs);
protected:
	GtGC *next;
	GtDisplay *dsp;
	GtDrawable *dr;
	GtPixmap *tile,*stip,*cmask;
	GtFont *font;
	Display *xdsp;
	GC gc;
	int cnt;
};

extern GtGC *GtMkGC(GtDisplay *d,GtTag *tgs);
extern GtGC *GtMkGC(GtDisplay *d,ulong tag,...);
extern GtGC *GtMkSharedGC(GtDisplay *d,GtTag *tgs);
extern GtGC *GtMkSharedGC(GtDisplay *d,ulong tag,...);

#endif /* INC_GTGC_H */
